<?php
/**
 * Manages Manialink pages.
 * Handles callbacks for actions and links every
 * action with a unique id.
 *
 * @author Florian Schnell
 */
class ManialinkFramework {
	private $_topical;
	private $_callbacks;
	private $_window_handle;
	private $_windows;
	private $_action_handle;
	private $_actions;
	private $_minimized;
	private $_debug;
	private $_xml;
	private $_wins_per_task;
	private $_recent_minimized;
	private $_maximized;
	private $_events;
	private $_owner;
	private $_deleted_windows;		// 08-2-10 Assembler - keep track of deleted windows so we can paint a blank manialink to clear them
	private $_ui_notice;			// 08-2-10 Assembler - 8 Custom UI elements that can be turned on/off, default is on for each
	private $_ui_challenge_info;
	private $_ui_net_infos;
	private $_ui_chat;
	private $_ui_checkpoint_list;
	private $_ui_round_scores;
	private $_ui_scoretable;
	private $_ui_global;
	public $_show_minimized;

	function __construct() {
		$this->_window_handle = 0;
		$this->_windows = array();
		$this->_callbacks = array();
		$this->_action_handle = 0;
		$this->_topical = false;
		$this->_minimized = 0;
		$this->_actions = array();
		$this->_debug = false;
		$this->_wins_per_task = 8;
		$this->_deleted_windows = array();
		$this->_ui_notice = true;				// avatar and line of text, upper left of screen
		$this->_ui_challenge_info = true;		// current name of track, upper right
		$this->_ui_net_infos = true;			// Player Pos/Count, left side of screen - not currently working
		$this->_ui_chat = true;					// the entire chat window and chat input line
		$this->_ui_checkpoint_list = true;		// list of players and how far off their checkpoint you are - lower right, to the left of the speed/meters driven
		$this->_ui_round_scores = true;			// player scores in rounds mode
		$this->_ui_scoretable = true;			// score table at end of single lap race (on a non-lap based track)
		$this->_ui_global = true;				// turn off everything
		$this->_show_minimized = true;
	}

	function dispose() {
		for ($i = 0; $i < count($this->_windows); $i++) {
			if (isset($this->_windows[$i])) {
				$this->_windows[$i]->dispose();
				unset($this->_windows[$i]);
			}
		}
		unset($this->_callbacks);
		unset($this->_windows);
		unset($this->_owner);
		unset($this->_actions);
		unset($this->_events);
		unset($this->_recent_minimized);
		unset($this->_xml);
	}

	function __destruct() {
		if ($this->_debug) echo "destroying manialink framework ...\n";
	}

	/**
	 * A new window has been minimized.
	 * Returns the new tab id.
	 *
	 * @return int
	 */
	public function addWindowToTab() {
		return $this->_minimized++;
	}

	/**
	 * Window has been removed from the tab.
	 * Returns the new tab id.
	 *
	 * @return int
	 */
	public function removeWindowFromTab() {
		$i = 0;
		foreach ($this->_windows as $window) {
			if ($window->isStatic()) next;
			if ($window->isMinimized()) {
				$window->setTabPos($i++);
			}
		}
		return $this->_minimized--;
	}

	/**
	 * Minimizes all windows and puts them
	 * into the tab bar.
	 *
	 */
	public function minimizeAllWindows() {
		$this->_minimized = 0;
		foreach ($this->_windows as $window) {
			if (!$window->isMinimized()) $this->_recent_minimized = $window;
			$window->minimize();
		}
	}

	/**
	 * Hide/Show all windows that are minimized - for intermission between tracks
	 */
	public function setShowMinimizedWindows($show = true) {
		$this->_show_minimized = $show;
	}

	/**
	 * Adds a new window to the framework.
	 * The currently open window will be minimized.
	 * Return the id
	 *
	 * @param string $title
	 * @param float $width
	 * @param float $height
	 * @param object $plugin
	 * @return ManialinkPage
	 */
	public function createWindow($title, $width = 0, $height = 0, $plugin = null, $is_static = false) {

		$this->_topical = false;

		// create a new manialink page ...
		$window = new ManialinkPage($this, $title, $width, $height, $plugin, $is_static);

		// set page handler ...
		$window->setId($this->_window_handle++);

		// minimize all open windows ...
		if (!$is_static) $this->minimizeAllWindows();

		// add page to the intern page list ...
		$this->_windows[$window->getId()] = $window;

		if ($this->_debug) echo "created window: " . $window->getId() . LF;

		// return the window object ...
		return $window;
	}

	/**
	 * Adds a new window to the framework.
	 * The currently open window will be minimized.
	 * Return the id
	 *
	 * @param string $title
	 * @param string $message
	 * @param int $timeout
	 * @return ManialinkPage
	 */
	public function createPopup($title, $message, $timeout = 0) {

		// count # of lines in the msg, + 3, * .04 = height

		$h = substr_count($message, LF);
		$h += 3;

		$h *= .04;

		// create window ...
		$window = $this->createWindow($title, 0.5, $h);

		// resize it ...
		$window->setWidth(1);

		// now set the content ...
		$window->setContent('<line><cell><text>'.$message.'</text></cell></line>');

		// set the window's timeout ...
		$window->setTimeout($timeout);

		// and return the window object ...
		return $window;
	}

	/**
	 * Removes timedout popups from the framework.
	 */
	public function closePopups() {
		foreach ($this->_windows as $window) {
			if ($window->hasTimeout()) {
				if ($window->isTimedout()) {
					$this->deleteWindow($window->getId());
					$this->openRecentMinimizedWindow();
				}
			}
		}
	}

	/**
	 * Removes a page from the framework.
	 *
	 * @param int $handle
	 */
	public function deleteWindow($handle) {

		// if there's a page with that handle ...
		if (isset($this->_windows[$handle])) {

		$this->_deleted_windows[] = $handle;

			if ($this->_debug) echo "removed window: ".$handle.LF;

			// then remove it ...
			$this->_windows[$handle]->dispose();
			unset($this->_windows[$handle]);

			// framework code changed ...
			$this->invalidate();

			// ok!
			return true;
		} else {

			// otherwise return false ...
			return false;
		}
	}

	/**
	 * Can be called to execute all callbacks
	 * that were registered for a specific action.
	 *
	 * @param int $action_id
	 * @param mixed $params
	 */
	function releaseWindowEvent($action_id, $player) {

		if ($this->_debug)  echo "incoming event: ".$action_id.LF;

		// go through the list of windows ...
		foreach ($this->_windows as $window) {

			// cancel if action does not exist ...
			if(!isset($this->_actions[$window->getId()][$action_id])) continue;

			// get name to this action id ...
			$action_name = $this->_actions[$window->getId()][$action_id]['name'];
			$action_param = $this->_actions[$window->getId()][$action_id]['param'];

			// get the array of callbacks this window has for this action ...
			$callbacks = $window->getCallbacks($action_name);

			if ( isset($callbacks) ) {		// are any callbacks set?
				// process callbacks for this window ...
				foreach ($callbacks as $callback) {
					$func_params = array($action_param, $window, $player);

					// if an callback object is set ...
					if (isset($callback['obj'])) {

						// check if this object has such a function and if, then call it ...
						if (!method_exists($callback['obj'], $callback['func'])) return;
						call_user_func_array(array($callback['obj'], $callback['func']), $func_params);
					} else {

						// check if the function exists and if, then call it ...
						if (!function_exists($callback['func'])) return;
						call_user_func_array($callback['func'], $func_params);
					}
				}
			} else {
				echo 'Developer error - no callback defined for ' . $action_name . CRLF;
			}
		}
	}

	private function bool2text($boolval)
		{
		if ( $boolval )
			return 'true';
		else
			return 'false';
		}  //  bool2text

	/**
	 * Updates all action ids.
	 * Returns the xml data of the whole
	 * framework.
	 *
	 * @return string
	 */
	public function getXml() {

		if ($this->_silent) return '';

		$this->parsePageActions();

		$xml = '<?xml version="1.0" encoding="utf-8" ?>';

	$xml .= '<manialinks>';

/*	$this->_ui_notice = false;
	$this->_ui_challenge_info = true;
	$this->_ui_net_infos = false;
	$this->_ui_chat = false;
	$this->_ui_checkpoint_list = false;
	$this->_ui_round_scores = false;
	$this->_ui_scoretable = false;
	$this->_ui_global = false;
*/
	$xml .= '<custom_ui>
		<notice visible="' . $this->bool2text($this->_ui_notice) . '"/>
		<challenge_info visible="' . $this->bool2text($this->_ui_challenge_info) . '"/>
		<net_infos visible="' . $this->bool2text($this->_ui_net_infos) . '"/>
		<chat visible="' . $this->bool2text($this->_ui_chat) . '"/>
		<checkpoint_list visible="' . $this->bool2text($this->_ui_checkpoint_list) . '"/>
		<round_scores visible="' . $this->bool2text($this->_ui_round_scores) . '"/>
		<scoretable visible="' . $this->bool2text($this->_ui_scoretable) . '"/>
		<global visible="' . $this->bool2text($this->_ui_global) . '"/>
		</custom_ui>';


		$xml .= $this->_xml;

		if ($this->_minimized > 0)
			$height = 0.05 + (floor(($this->_minimized - 1) / $this->getWinsPerTask()) * 0.045);
		else
			$height = 0.05;

	foreach ($this->_deleted_windows as $handle)
		$xml .= '<manialink id="' . $handle . '"></manialink>';

	$this->_deleted_windows = array();

		$xml .= '</manialinks>';

//echo $xml.CRLF;

	return $xml;
	}

	/**
	 * Returns whether the client's version
	 * of the framework is still uptodate.
	 *
	 * @return bool
	 */
	public function isValid() {

		// check if an update is needed ...
		return $this->_topical;
	}

	/**
	 * Marks the framework as uptodate.
	 *
	 */
	public function validate() {

		// aseco's version of the framework is uptodate ...
		$this->_topical = true;
	}

	/**
	 * Framework code has changed.
	 * An update on the client machines is needed.
	 *
	 */
	public function invalidate() {

		// makes aseco refresh the player's framework ...
		$this->_topical = false;
	}

	/**
	 * Replaces all actions with Ids.
	 *
	 */
	private function parsePageActions() {

		// reset the action handle ...
		$this->_action_handle = 0;
		$this->_actions = array();
		$this->_xml = '';

		// parse each page that is in the frameset ...
		foreach ($this->_windows as $window) {

			$xml = $window->getXml();
		// 08-2-25 AM - only search/replace items in the xml for a single window at a time,
			// now search for actions that have to be replaced with ids ...
			$matches = array();
			preg_match_all('/\{name:(.[^\}]*),param:(.[^\}]*)\}/', $xml, $matches, PREG_SET_ORDER);

			// replace every found action ...
			foreach ($matches as $match) {
				$name = $match[1];
				$param = $match[2];

				// register the new action ...
				$this->_actions[$window->getId()][$this->_action_handle]['name'] = $name;
				$this->_actions[$window->getId()][$this->_action_handle]['param'] = $param;

				// replace action with the generated action id ...
				$xml = str_replace('{name:'.$name.',param:'.$param.'}', $this->_action_handle++, $xml);
			}

			$this->_xml .= $xml;
		}
	}

	public function getWinsPerTask() {
		return $this->_wins_per_task;
	}

	public function setMaximized($in_maximized) {
		$this->_maximized = $in_maximized;
	}

	public function refreshTab() {
		$this->minimizeAllWindows();
		if (isset($this->_maximized)) $this->_maximized->maximize();
	}

	public function getHeight() {
		if ($this->_minimized > 0)
			return 0.05 + (floor(($this->_minimized - 1) / $this->getWinsPerTask()) * 0.045);
		else
			return 0.05;
	}

	public function openRecentMinimizedWindow() {
		// try to open the recent minimized window ...
		if (isset($this->_recent_minimized) && isset($this->_windows[$this->_recent_minimized->getId()])) {
			$this->_recent_minimized->maximize();
		} else {
			$this->refreshTab();
		}
	}

	public function addEvent($name, $callback_obj, $callback_func) {
		$event['obj'] = $callback_obj;
		$event['func'] = $callback_func;
		$this->_events[$name][] = $event;
	}

	public function releaseEvent($name) {
		if (!isset($this->_events[$name])) return;
		foreach ($this->_events[$name] as $event) {
			if (!method_exists($event['obj'], $event['func'])) return;
			call_user_func_array(array(&$event['obj'], $event['func']), array($this));
		}
	}

	function setOwner($player) {
		$this->_owner = $player;
	}

	function getOwner() {
		return $this->_owner;
	}

	public function setUIChat($newstate)
		{
	// the entire chat window
	$this->_ui_chat = $newstate;
	$this->invalidate();
		}

	public function setUIChallengeInfo($newstate)
		{
	// name of track in upper right
	$this->_ui_challenge_info = $newstate;
	$this->invalidate();
		}

	public function setUINetInfos($newstate)
		{
	// no freaking clue - should have been the network graph and stuff
	$this->_ui_net_infos = $newstate;
	$this->invalidate();
		}

	public function setUINotice($newstate)
		{
	// notices about who takes first and such - top line of screen, left justified with player avatar followed by text
		$this->_ui_notice = $newstate;
	$this->invalidate();
		}

	public function setUICheckpointList($newstate)
		{
	// list of players and the times diff in lower right of screen
	$this->_ui_checkpoint_list = $newstate;
	$this->invalidate();
		}

	public function setUIRoundScores($newstate)
		{
	// scores for rounds mode on right side of screen
	$this->_ui_round_scores = $newstate;
	$this->invalidate();
		}

	public function setUIScoreTable($newstate)
		{
	// popup at end of each run, if it's not a lap based track
	$this->_ui_scoretable = $newstate;
	$this->invalidate();
		}

	public function setUIGlobal($newstate)
		{
	// hide everything
	$this->_ui_global = $newstate;
	$this->invalidate();
		}
}
?>
