<?php
class ManialinkPage {

  private $_framework;
  private $_id;
  private $_content;
  private $_pos_x;
  private $_pos_y;
  private $_title;
  private $_visible;
  private $_minimized;
  private $_callbacks;
  private $_tabpos;
  private $_plugin;
  private $_xml;
  private $_cur_page;
  private $_pages;
  private $_width;
  private $_height;
  private $_is_static;
  private $_timeout;
  private $_remaining_time;
  private $_background;
  private $_textsize;
  private $_bordersize;
  private $_title_bgcolor;

  public function __construct($framework, $title, $width, $height, $plugin, $is_static = false, $theme = 'default') {
  	$this->_framework = $framework;
  	$this->_title = $title;
  	$this->_plugin = $plugin;
  	$this->_content = '';
  	$this->_visible = true;
  	$this->_cur_page = 0;
  	$this->_offset = 0;
  	$this->setContent("");
  	$this->clearCallbacks();
  	$this->_is_static = $is_static;
  	$this->_timeout = 0;

    // for static windows,
    // treat the width and height params as posx and posy ...
    if ($this->_is_static) {
  	  $this->_pos_x = $width;
      $this->_pos_y = $height;
    // non-static ones are placed into the screen center ...
    } else {
      $this->_pos_x = $width/2;
      $this->_pos_y = $height/2;
    }

  	// future themes support ...
  	switch ($theme) {
  		default:
    		$this->_width = $width;
    		$this->_height = $height;
    		$this->_background = 'fffe';
    		$this->_textsize = 2;
    		$this->_bordersize = 0.01;
    		$this->_title_bgcolor = '000f';
  			break;
  	}
  }

  function dispose() {
    unset($this->_callbacks);
    unset($this->_framework);
    unset($this->_plugin);
  }

  function __destruct() {
    // echo "destroying manialink page '".$this->_title."' ...\n";
  }

  /**
   * Adds a new callback to this window.
   * Every action can have unlimited callback functions
   * that will be executed when the text with the
   * specified action is clicked.
   *
   * @param string $action_name
   * @param string $callback_func
   * @param mixed $callback_obj
   */
  public function addCallback($action_name, $callback_func, $callback_obj = null) {

    // create callback ...
    $callback = array();

    // set callback properties ...
    $callback['obj'] = $this->_plugin;
    $callback['func'] = $callback_func;
    if (isset($callback_obj)) $callback['obj'] = $callback_obj;

    // register the new action callback ...
    $this->_callbacks[$action_name][] = $callback;
  }

  /**
   * Gets an array of all callbacks that
   * are linked to a specified action with this window.
   *
   * @param string $action_name
   * @return array
   */
  public function getCallbacks($action_name) {
    return $this->_callbacks[$action_name];
  }

  /**
   * Removes all callbacks.
   * Except the window specific ones.
   *
   */
  public function clearCallbacks() {

    // remove all callbacks ...
    $this->callbacks = array();

    // add standard page events ...
    $this->addCallback('close', 'close', $this);
    $this->addCallback('max', 'maximize', $this);
    $this->addCallback('min', 'minimize', $this);
    $this->addCallback('next', 'nextPage', $this);
    $this->addCallback('prev', 'prevPage', $this);
  }

  /**
   * Moves to the next page of this window.
   * Only available if the page content is an array.
   *
   * @return bool
   */
  public function nextPage() {
    if (!is_array($this->_content)) return false;
    $this->_cur_page++;
    $this->_framework->invalidate();
    return true;
  }

  /**
   * Moves to the previous page of this window.
   * Only available if the page content is an array.
   *
   * @return bool
   */
  public function prevPage() {
    if (!is_array($this->_content)) return false;
    $this->_cur_page--;
    $this->_framework->invalidate();
    return true;
  }

  /**
   * Closes this window.
   * It will be removed from the framework, that
   * makes it unrecoverable.
   *
   */
  public function close() {
    if (isset($this->_framework))
      $this->_framework->deleteWindow($this->getId());
  }

  /**
   * Hides this window.
   * This keeps it recoverable.
   *
   */
  public function hide() {
    $this->_visible = false;
    $this->_framework->invalidate();
  }

  /**
   * Recovers a hidden window.
   *
   */
  public function show() {
    $this->_visible = true;
    $this->_framework->invalidate();
  }

  /**
   * Gets a window from the tab back
   * to the front of the screen.
   * Currently opened window(s) will be put into
   * the tab.
   *
   */
  public function maximize() {
    if ($this->_is_static) return;
    $this->_framework->minimizeAllWindows();
    $this->_minimized = false;
    $this->_framework->removeWindowFromTab();
    $this->_framework->setMaximized($this);
    $this->_framework->invalidate();
    $this->_framework->releaseEvent('onMaximize');
  }

  /**
   * Puts the window to the tab.
   *
   */
  public function minimize() {
    if ($this->_is_static) return;
    $this->_minimized = true;
    $this->_tabpos = $this->_framework->addWindowToTab();
    $this->_framework->invalidate();
    $this->_framework->releaseEvent('onMinimize');
  }

  /**
   * Checks if the page is minimized.
   *
   * @return bool
   */
  public function isMinimized() {
    return $this->_minimized;
  }

  /**
   * Sets a new tab id, this will change
   * the window position in the tab bar.
   *
   * @param int $newpos
   */
  public function setTabPos($newpos) {
    $this->_tabpos = $newpos;
  }

  /**
   * Changes the page content.
   * You can apply either a string (single page window)
   * or an array (multi page window - interface will be created)
   *
   * @param mixed $content
   */
  public function setContent($content) {

    // set the new content ...
    $this->_content = $content;

    // invalidate the parent framework so it is rebuild ...
    $this->_framework->invalidate();
  }

  /**
   * Returns the current page content.
   *
   * @return mixed
   */
  public function getContent() {
    return $this->_content;
  }

  /**
   * Sets a new horizontal page position.
   *
   * @param float $posx
   */
  public function setPosX($posx) {
    $this->_pos_x = $posx;
    if ($posx > 0.75) $this->_pos_y = 0.75;
    if ($posx < -0.75) $this->_pos_y = -0.75;
  }

  /**
   * Sets a new vertical page position.
   *
   * @param float $posy
   */
  public function setPosY($posy) {
    $this->_pos_y = $posy;
    if ($posy > 1) $this->_pos_y = 1;
    if ($posy < -1) $this->_pos_y = -1;
  }

  /**
   * Sets a new window width.
   *
   * @param float $width
   */
  public function setWidth($width) {
  	if (!$this->_is_static) {
      $this->_width = $width;
      if ($width > 2) $this->_width = 2;
      $this->_pos_x = $this->_width/2;
  	}
  }

  /**
   * Sets a new window height.
   *
   * @param float $height
   */
  public function setHeight($height) {
    if (!$this->_is_static) {
      $this->_height = $height;
      if ($heigh > 1.5) $this->_height = 1.5;
      $this->_pos_y = $this->_height/2;
    }
  }

  private function getButtonXml($caption) {
	$xml = '<cell></cell>';
  }

  public function getXml() {

    // page is hidden ...
    if (!$this->_visible) return '<manialink id="' . $this->_id . '"></manialink>';

	// page is minimized and we're not painting minimized windows right now
	if (!$this->_framework->_show_minimized && $this->_minimized && !$this->_is_static)
		{
		return '<manialink id="' . $this->_id . '"></manialink>';
		}

    // page is minimized ...
    if ($this->_minimized && !$this->_is_static) {
      $pos_x = (string)(0.995 - ($this->_tabpos % $this->_framework->getWinsPerTask()) * 0.21);
      $pos_y = "0.745" - (0.045 * floor($this->_tabpos / $this->_framework->getWinsPerTask()));
      $border = 0.005;
      $bg = 'fff8';
    } else {
      $pos_x = $this->_pos_x;
      $pos_y = $this->_pos_y;
      $border = $this->_bordersize;
      $bg = $this->_background;
      if (!$this->_is_static) $bg = '0000';
    }

    // build manialink page header ...
    $xml = '<manialink posx="' . $pos_x . '" posy="' . $pos_y . '" id="' . $this->_id . '">
		<type>default</type>
		<format textsize="'.$this->_textsize.'" textcolor="000f" />
		<background bgcolor="'.$bg.'" bgborderx="'.$border.'" bgbordery="'.$border.'" />';

    /**
     * Window is minimized
     */
    if ($this->_minimized && !$this->_is_static) {

      $title = substr($this->_title, 0, 10);
      if (strlen($this->_title) > 10) $title .= '...';

		$xml .= "<quad size='0.19 0.05' posn='0 0' halign='left' valign='top' style='BgsPlayerCard' substyle='ProgressBar' action='{name:max,param:0}'/>
  				<label textsize='2' textcolor='fff' posn='1.1 -.46' halign='left' valign='top' text='$title' />";

  	/**
     * Window is opened
     */
    } else {

      $multipage = '';
      if (is_array($this->_content)) {
        $multipage = ' ('.($this->_cur_page+1).'/'.count($this->_content).')';
      }

      $timeout = '';
      if ($this->_timeout > 0) {
        $timeout = ' - closing in '.$this->_remaining_time.' sec';
      }

      if (!$this->_is_static) {

      	// make background ...
        $xml .= "<quad size='".($this->_width+$border*2)." ".($this->_height+$border*2)."' pos='$border $border' halign='left' valign='top' style='Bgs1InRace' substyle='BgButtonBig' />";

        // make title-bar ...
        $xml .= '<line height="0.06"><cell /></line>'; // compability spacer for old <lines>-designs ...
        $xml .= '<quad size="'.($this->_width-0.09).' 0.05" pos="0 0 -.05" halign="left" valign="top" style="Bgs1InRace" substyle="BgPager"/>';
        $xml .= '<label textsize="2" textcolor="fff" pos="-0.016 -0.01 -.1" halign="left" valign="top" text="'.$this->_title.$multipage.$timeout.'" />';

      	// creat min button ...
      	$min_x = -$this->_width+0.05;
      	$min_y = -0.005;
      	$xml .= '<quad size="0.04 0.04" pos="'.$min_x.' '.$min_y.' -.1" halign="right" valign="top" style="Icons64x64_1" substyle="ArrowUp" action="{name:min,param:0}"/>';
//      	$xml .= '<quad size="0.04 0.04" pos="'.$min_x.' '.$min_y.' -.1" halign="right" valign="top" style="BgsPlayerCard" substyle="BgActivePlayerCard" action="{name:min,param:0}"/>';
//      	$xml .= '<label textsize="2" textcolor="000" pos="'.($min_x+0.01).' '.($min_y+0.01).' -.11" halign="right" valign="top" text="_" />';

      	// create close button ...
      	$close_x = -$this->_width;
      	$close_y = 0;
      	$xml .= '<quad size="0.05 0.05" pos="'.$close_x.' '.$close_y.' -.1" halign="right" valign="top" style="Icons64x64_1" substyle="Close" action="{name:close,param:0}"/>';
//      	$xml .= '<quad size="0.05 0.05" pos="'.$close_x.' '.$close_y.' -.1" halign="right" valign="top" style="Icons64BgsPlayerCard" substyle="BgActivePlayerCard" action="{name:close,param:0}"/>';
//      	$xml .= '<label textsize="3" textcolor="000" pos="'.($close_x+0.015).' '.$close_y.' -.11" halign="right" valign="top" text="x" />';

      }

      // page body ...
      if (is_array($this->_content)) {

        // get current page ...
        $xml .= $this->_content[$this->_cur_page];

        // make close button ...
        $cls_btn_x = (-$this->_width/2)+0.07;
        $btns_y = -$this->_height+0.01;
        $xml .= '<quad size="0.14 0.05" pos="'.$cls_btn_x.' '.$btns_y.' -.1" halign="left" valign="bottom" style="BgsPlayerCard" substyle="BgActivePlayerCard" action="{name:close,param:0}"/>';
      	$xml .= '<label textsize="2" textcolor="000" pos="'.($cls_btn_x-0.04).' '.($btns_y+0.005).' -.11" halign="left" valign="bottom" text="close" />';

      	// show previous button if necessary ...
      	if (isset($this->_content[$this->_cur_page-1])) {
      	  $prv_btn_x = -0.01;
      	  $xml .= '<quad size="0.05 0.05" pos="'.$prv_btn_x.' '.$btns_y.' -.1" halign="left" valign="bottom" style="Icons64x64_1" substyle="ArrowPrev" action="{name:prev,param:0}"/>';
      	}

      	// show next button if necessary ...
        if (isset($this->_content[$this->_cur_page+1])) {
      	  $nxt_btn_x = -$this->_width+0.01;
          $xml .= '<quad size="0.05 0.05" pos="'.$nxt_btn_x.' '.$btns_y.' -.1" halign="right" valign="bottom" style="Icons64x64_1" substyle="ArrowNext" action="{name:next,param:0}"/>';
      	}

      } else {
        $xml .= $this->_content;
      }
    }

    // build window footer ...
    $xml .= '</manialink>';

    return $xml;
  }

  /**
   * Sets custom window background
   *
   * @param string $in_color
   */
  public function setBackground($in_color) {
    $this->_background = $in_color;
  }

  /**
   * Gets custom window background
   */
  public function getBackground() {
    return $this->_background;
  }

  /**
   * Sets custom window border size
   *
   * @param double $in_size
   */
  public function setBorderSize($in_size) {
    $this->_bordersize = $in_size;
  }

  /**
   * Returns custom window border size
   *
   * @return double
   */
  public function getBorderSize() {
    return $this->_bordersize;
  }

  /**
   * Sets custom window textsize
   *
   * @param int $in_size
   */
  public function setTextsize($in_size) {
    $this->_textsize = $in_size;
  }

  /**
   * Sets a new window id
   */
  public function setId($in_id) {
    if (!isset($this->_id)) {
      $this->_id = $in_id;
      return true;
    } else {
      return false;
    }
  }

  /**
   * Gets the window's id
   */
  public function getId() {
    return $this->_id;
  }

  /**
   * Makes a window static.
   * i.e.: this window is not managed by the
   * Manialink framework.
   */
  public function makeStatic() {
    $this->_is_static = true;
  }

  /**
   * Makes a window unstatic.
   * The window is managed by the framework again.
   */
  public function makeUnstatic() {
    $this->_is_static = false;
  }

  /**
   * Checks whether the window is managed by the
   * framework or not.
   */
  public function isStatic() {
    return $this->_is_static;
  }

  /**
   * Returns the title of the current window.
   */
  public function getTitle() {
    return $this->_title;
  }

  /**
   * Sets the title for the current window.
   */
  public function setTitle($in_title) {
    $this->_title = $in_title;
  }

  public function setTimeout($in_timeout) {
    if ( $in_timeout > 0 )
      $this->_timeout = time() + $in_timeout;
    else
      $this->_timeout = 0;
  }

  public function hasTimeout() {
    return ($this->_timeout > 0);
  }

  public function isTimedout() {

    // don't do anything if window doesn't have a timeout value set ...
    if ($this->_timeout <= 0) return false;

    // buffer old remaining time ...
    $old_remaining_time = $this->_remaining_time;

    // calculate new remaining time ...
    $this->_remaining_time = $this->_timeout - time();

    // refresh the window if remaining time has changed ...
    if ($old_remaining_time != $this->_remaining_time) $this->_framework->invalidate();

    // return whether the window has timed out ...
    return ($this->_remaining_time < 0);
  }

  /**
   * @return ManialinkFramework
   */
  public function getFramework() {
    return $this->_framework;
  }
}
?>
